@@ -10,6 +10,12 @@ NULL
 #'  (ceramic) types.
 #' @slot weights An \eqn{m \times p}{m x p} [`integer`] [`matrix`] giving the
 #'  weights used.
+#' @slot simulation A three columns [`numeric`] matrix giving the summary
+#'  statistics of the simulated dates (`mean` and `lower` and `upper` boundaries
+#'  of the confidence interval).
+#' @param level A length-one [`numeric`] vector giving the confidence level.
+#' @slot replications An [`integer`] giving the number of bootstrap
+#'  replications.
 #' @section Coerce:
 #'  In the code snippets below, `x` is a `MeanDate` object.
 #'  \describe{
@@ -30,6 +36,16 @@ NULL
   contains = "numeric"
 )
 
+.SimulationMeanDate <- setClass(
+  Class = "SimulationMeanDate",
+  slots = c(
+    simulation = "matrix",
+    level = "numeric",
+    replications = "integer"
+  ),
+  contains = "MeanDate"
+)
+
 # EventDate ====================================================================
 #' Date Model
 #'
@@ -154,6 +170,9 @@ NULL
 #' @slot parameter An [`integer`] giving the degrees of freedom for the
 #'  t-statistic.
 #' @slot p_value A [`numeric`] vector giving the the p-value for the test.
+#' @param x An object from which to extract element(s) or in which to replace
+#'  element(s).
+#' @param i A [`character`] string specifying elements to extract.
 #' @section Coerce:
 #'  In the code snippets below, `x` is an `IncrementTest` object.
 #'  \describe{
@@ -173,3 +192,73 @@ NULL
     p_value = "numeric"
   )
 )
+
+# PermutationOrder =============================================================
+#' Permutation Order
+#'
+#' An S4 class to represent a permutation order.
+#' @slot rows_order An [`integer`] vector giving the rows permutation.
+#' @slot columns_order An [`integer`] vector giving the columns permutation.
+#' @slot method A [`character`] string indicating the seriation method used.
+#' @param x An object from which to extract element(s) or in which to replace
+#'  element(s).
+#' @param i A [`character`] string specifying elements to extract.
+#' @seealso [`dimensio::CA-class`]
+#' @author N. Frerebeau
+#' @family classes
+#' @docType class
+#' @aliases PermutationOrder-class
+.PermutationOrder <- setClass(
+  Class = "PermutationOrder",
+  slots = c(
+    rows_order = "integer",
+    columns_order = "integer"
+  ),
+  contains = "VIRTUAL"
+)
+
+#' @rdname PermutationOrder-class
+#' @aliases RankPermutationOrder-class
+.RankPermutationOrder <- setClass(
+  Class = "RankPermutationOrder",
+  contains = "PermutationOrder"
+)
+
+#' @rdname PermutationOrder-class
+#' @aliases AveragePermutationOrder-class
+.AveragePermutationOrder <- setClass(
+  Class = "AveragePermutationOrder",
+  contains = c("PermutationOrder", "CA")
+)
+
+# RefineCA =====================================================================
+#' Partial Bootstrap CA
+#'
+#' An S4 class to store partial bootstrap correspondence analysis results.
+#' @slot hull A three columns [`numeric`] matrix giving the vertices
+#'  coordinates (`x`, `y`) of the convex hull and a identifier (`id`)
+#'  to link each row to a variable.
+#' @slot length A [`numeric`] vector giving the convex hull maximum
+#'  dimension length.
+#' @slot keep An [`integer`] vector giving the subscript of the variables
+#'  to be kept.
+#' @slot cutoff A length-one [`numeric`] vector giving the cutoff value for
+#'  samples selection.
+#' @param x An object from which to extract element(s) or in which to replace
+#'  element(s).
+#' @param i A [`character`] string specifying elements to extract.
+#' @seealso [`dimensio::CA-class`]
+#' @author N. Frerebeau
+#' @family classes
+#' @docType class
+#' @aliases RefineCA-class
+.RefineCA <- setClass(
+  Class = "RefineCA",
+  slots = c(
+    hull = "matrix",
+    length = "numeric",
+    keep = "integer",
+    cutoff = "numeric",
+    margin = "integer"
+  )
+)
